#!/usr/bin/perl -I/home/pinson/szpari/programs/Perl/

print STDERR "
                    :-] G R O M A C S [-:
                        helpful tools

                     :-] g_s2anal.pl  [-:

Option     Filename  Type          Description
------------------------------------------------------------
";


use Getopt::Std;

$opt_f="md0.log";
$opt_r="s2rho.xvg";
$opt_c="s2correl.xvg";
$opt_o="s2snapshots.xvg";
$opt_e="";
$opt_t="NH";
$opt_s=1000;

getopts('f:o:r:c:e:s:t:ghv') || die "ERROR in options, exiting\n";

$S2file=$opt_s;

if ($opt_h){$HO="yes"} else{$HO="no"}
if($opt_v){$V=1;$VO="yes"}else{$VO="no"}
if ($opt_g){$GO="yes"} else{$GO="no"}

printf STDERR ("  -f  %12s   Input         MD log file\n",$opt_f);
printf STDERR ("  -o  %12s   Output        xmgr/xvg file with s2 values per selected frames\n",$opt_o);
printf STDERR ("  -r  %12s   Output        xmgr/xvg file with rho\n",$opt_r);
printf STDERR ("  -c  %12s   Output        xmgr/xvg file with sim/calc S2 correlation\n",$opt_c);
printf STDERR ("  -e  %12s   Input, Opt.   s2res file with experimental S2 values\n",$opt_e);

print STDERR"
      Option   Type  Value  Description
------------------------------------------------------
";

printf STDERR ("          -h   bool %9s  Print help info and quit\n",$HO);
printf STDERR ("          -v   bool %9s  Be loud and noisy\n",$VO);
printf STDERR ("          -s   int  %9s  Only write output at every nr-th step\n",$opt_s);
printf STDERR ("          -t   enum %9s  S2 type to include in output (use with -e)\n",$opt_t);
printf STDERR ("          -g   bool %9s  Generate gnuplot .dat and .gpl files\n",$GO);


if ($opt_h) {
print STDERR "

Reads the md log file specified and gets S2 vakues from the lines starting with
'S2RES'.
Generates 3 outout files by default:
-  s2snapshots.xvg contains the experimental S2 data for each 
   atom pair plus the experimental ones per frames selected.
   if an s2res file is specified with -e, the experimental errors are
   included in the output file for the selected S2 type.
-  s2rho.xvg contains the rho-value changing in the course of the simulation. 
-  s2correl.xvg contains the correlation between experimental and simulated S2 values 
   changing in the course of the simulation. 

Supply 'NONE' to -o or -r to suppress the generation of one of these files.
If -g is specified, the program also generates files suitable for viewing the data with
gnuplot: <filename>.dat contains the data in gnuplot-readable format and <filename>.gpl
contains the gnuplot commend sequence. <filename> are the base names for the two output 
.xvg files.
Launch gnuplot with the command 
 'gnuplot <filename>.gpl -' 
to view the diagrams generated.

";
exit;
}


if ($opt_e ne ""){
   open (EF,"$opt_e") || die "FATAL: Cannot open experimental data file $opt_e\n";
   $pair=0;
   while(<EF>){
     chop;
     $_=~s/^ +//;
     next if ($_ !~ /[0-9]/); # skip blank lines
     ($rn1,$atom1,$rn2,$atom2,$s2,$s2err,$bl)=split(/ +/,$_);
     $type1=$atom1.$atom2;
     $type2=$atom2.$atom1;  
     if (($type1 eq $opt_t) || ($type2 eq $opt_t)){
       $TYPEOK[$pair]=1;
     }
     $S2EXP[$pair]=$s2;
     $S2EXP_ERR[$pair]=$s2err;
     $pair++;
   }
   close(EF);
}


# Reading in md.log file
open (MDLIN,"$opt_f") || die "FATAL: Log file $opt_f not found!\n";
$frame=-1;
$laststep=-1;
while(<MDLIN>){
    chop;
    if ($_ =~ /^S2RES/){
	@c=split(/ +/,$_);
	$step=$c[2];
	$pair=$c[7];
	$foundpair{$pair}=1;

	if ($step ne $laststep){$frame++}
	$STEP[$frame]=$step;
	$S2SIM[$frame][$pair]=$c[13];
        # if -e not specified, take experimental values from log file (first frame)
	if (($frame == 0) && ($opt_e eq "")){$S2EXP[$pair]=$c[15];}

	$laststep=$step;
	print STDERR "Read: $_\r" if $V;
    }

}
close(MDLIN);


if ($opt_o ne "NONE"){
 if (-e $opt_o){
   print STDERR "Back off! Backing up $opt_o to \#$opt_o\#\n";
   system "mv $opt_o \#$opt_o\#";
 }
 open (OF,">$opt_o") || die "Cannot create $opt_o\n";
 print OF "# S2 values of all pairs during the simulation\n";
 foreach $pair (sort {$a <=> $b} keys %foundpair){
  if (($opt_e eq "") || ($TYPEOK[$pair])){

   printf OF ("%4d %7.3f",$pair,$S2EXP[$pair]);
   if ($opt_e ne ""){
       printf OF (" %7.3f",$S2EXP_ERR[$pair]);
   }
   for ($f=0; $f < $frame; $f++){
     if (($opt_s == 0) || (($STEP[$f]  % $opt_s) == 0)){ 
        printf OF (" %7.3f",$S2SIM[$f][$pair]);
     }
   }
   printf OF "\n"; 
  }# ir pair OK
 }
close(OF);

 if ($opt_g){
   if ($opt_o !~ /\.xvg$/){
       print STDERR "Output file $opt_o has no .xvg extension: appending .dat/.gpl to full filename\n";
       $datfile=$opt_o.".dat";
   }
   else{
       $datfile=$opt_o;
       $datfile=~s/\.xvg$/.dat/;
   }
   system("sed 's/^\@/\#\@/' < $opt_o > $datfile");
   $gplfile=$datfile;
   $gplfile=~s/\.dat/.gpl/;
   open(GF,">$gplfile") || die "Cannot create $gplfile\n";
   printf GF ("plot [][0:1] '$datfile' using 1:2 title 'exp' w lp lw 4");
   $s=3;
   if ($opt_e ne ""){
       printf GF (", '$datfile' using 1:2:3 notitle w errorbars 1");
       $s=4;
   }


   for ($f=0; $f < $frame; $f++){
     if (($opt_s == 0) || (($STEP[$f]  % $opt_s) == 0)){
	 $flgnd="step".$STEP[$f];
	 printf GF (", '$datfile' using 1:%d title '%s' w l",$s,$flgnd);
	 $s++;
     }
   }
   printf GF ";\n";  
   close(GF);

 }#_if $opt_g

}#_ if $opt_o ne NONE

if ($opt_r ne "NONE"){
 if (-e $opt_r){
   print STDERR "Back off! Backing up $opt_r to \#$opt_r\#\n";
   system "mv $opt_r \#$opt_r\#";
 }
 open (RF,">$opt_r") || die "Cannot create $opt_r\n";
 print RF "# Changes in Rho during the simulation\n";

 for ($f=0; $f < $frame; $f++){
     if (($opt_s == 0) || (($STEP[$f]  % $opt_s) == 0)){ 
	 $rho=0;$npair=0;
	 foreach $pair (sort {$a <=> $b} keys %foundpair){
	   if (($opt_e eq "") || ($TYPEOK[$pair])){
	     #print STDERR "frame: $f pair: $pair EXP: $S2EXP[$pair]  SIM: $S2SIM[$f][$pair]\n" if $V;
	     $rho+=($S2EXP[$pair]-$S2SIM[$f][$pair])*($S2EXP[$pair]-$S2SIM[$f][$pair]);
	     $npair++;
	   }
	 }
	 $rho/=$npair;
	 printf RF ("%7d %7.3f\n",$STEP[$f],$rho);
     }
 }

close(RF);

 if ($opt_g){
   if ($opt_r !~ /\.xvg$/){
       print STDERR "Output file $opt_r has no .xvg extension: appending .dat/.gpl to full filename\n";
       $datfile=$opt_r.".dat";
   }
   else{
       $datfile=$opt_r;
       $datfile=~s/\.xvg$/.dat/;
   }
   system("sed 's/^\@/\#\@/' < $opt_r > $datfile");
   $gplfile=$datfile;
   $gplfile=~s/\.dat/.gpl/;
   open(GF,">$gplfile") || die "Cannot create $gplfile\n";
   printf GF ("plot  '$datfile' using 1:2 title 'rho' w l");
   printf GF ";\n";  
   close(GF);

 }#_if $opt_g

}#_ if $opt_r ne NONE

if ($opt_c ne "NONE"){
 if (-e $opt_c){
   print STDERR "Back off! Backing up $opt_c to \#$opt_c\#\n";
   system "mv $opt_c \#$opt_c\#";
 }
 open (CF,">$opt_c") || die "Cannot create $opt_c\n";
 print CF "# Changes in the correlation of simulated/experimental S2 values\n# during the simulation\n";

 for ($f=0; $f < $frame; $f++){
     if (($opt_s == 0) || (($STEP[$f]  % $opt_s) == 0)){ 
	 $npair=0;
	 @M=();@D=();
	 foreach $pair (sort {$a <=> $b} keys %foundpair){
	   if (($opt_e eq "") || ($TYPEOK[$pair])){
	     $M[1]+=$S2EXP[$pair];
	     $M[2]+=$S2SIM[$f][$pair];
	     $M[3]+=$S2EXP[$pair]*$S2SIM[$f][$pair];
	     $npair++;
	 }
	 }
	 if ($npair > 0){
	     $M[1]/=$npair;
	     $M[2]/=$npair;
	     $M[3]/=$npair;
	     foreach $pair (sort {$a <=> $b} keys %foundpair){
		 $D[1]+=($S2EXP[$pair]-$M[1])**2;
		 $D[2]+=($S2SIM[$f][$pair]-$M[2])**2;
		 #print STDERR "pair: $pair D1; $D[1] D2: $D[2] ($S2SIM[$f][$pair] - $M[2])\n";
	     }
	     $D[1]/=$npair;$D[1]=sqrt($D[1]);
	     $D[2]/=$npair;$D[2]=sqrt($D[2]);
	     if ($D[1]*$D[2] == 0){
		 printf CF ("%7d   NaN\n",$STEP[$f]);
	     }
	     else{
		 $R=($M[3]-($M[1]*$M[2]))/($D[1]*$D[2]);
		 printf CF ("%7d %7.3f\n",$STEP[$f],$R);
	     }
	 }
	 else{
	     printf CF ("%7d   NaN\n",$STEP[$f]);
	 }
     }
 }

close(CF);

 if ($opt_g){
   if ($opt_c !~ /\.xvg$/){
       print STDERR "Output file $opt_c has no .xvg extension: appending .dat/.gpl to full filename\n";
       $datfile=$opt_c.".dat";
   }
   else{
       $datfile=$opt_c;
       $datfile=~s/\.xvg$/.dat/;
   }
   system("sed 's/^\@/\#\@/' < $opt_c > $datfile");
   $gplfile=$datfile;
   $gplfile=~s/\.dat/.gpl/;
   open(GF,">$gplfile") || die "Cannot create $gplfile\n";
   printf GF ("plot '$datfile' using 1:2 title 'sim/exp S2 correlation' w l");
   printf GF ";\n";  
   close(GF);

 }#_if $opt_g

}#_ if $opt_c ne NONE
