#!/usr/bin/perl -I/home/pinson/szpari/programs/Perl/

print STDERR "
                    :-] G R O M A C S [-:
                        helpful tools

                 :-] gromacsres2top.pl  [-:

Option     Filename  Type          Description
------------------------------------------------------------
";


use Getopt::Std;

$opt_n="";
$opt_U=99.9;
$opt_d="";
$opt_s="";
$opt_N="";
$opt_D="";
$opt_S="";
$opt_p="topol.top";
$opt_o="topol_res.top";

getopts('n:N:ud:D:s:S:p:o:hv') || die "ERROR in options, exiting\n";

$nmrfile=$opt_n;
$dihedralfile=$opt_d;
$S2file=$opt_s;

if ($opt_u){$uout="yes"}
else{$uout="no"}

if ($opt_h){$HO="yes"} else{$HO="no"}
if($opt_v){$V=1;$VO="yes"}else{$VO="no"}

printf STDERR ("  -p  %12s   Input         Topology file\n",$opt_p);
printf STDERR ("  -o  %12s   Output        Topology file\n",$opt_o);
printf STDERR ("  -n  %12s   Input, Opt.   NMR distance restraint file (X-PLOR format)\n",$nmrfile);
printf STDERR ("  -d  %12s   Input, Opt.   Dihedral restraint file (BBM format)\n",$dihedralfile);
printf STDERR ("  -s  %12s   Input, Opt.   S2 restraint file (S2RES format)\n",$S2file);

print STDERR"
      Option   Type  Value  Description
------------------------------------------------------
";

printf STDERR ("          -h   bool %9s  Print help info and quit\n",$HO);
printf STDERR ("          -v   bool %9s  Be loud and noisy\n",$VO);
printf STDERR ("          -N   enum %9s  NOE restraint directive\n",$opt_N);
printf STDERR ("          -U   real %9.2f  Second upper limit (r2) for NOE restraints (nm)\n",$opt_U);
printf STDERR ("          -u   bool %9s  Use only unambiguous restraints in NOE list\n",$uout);
printf STDERR ("          -D   enum %9s  Dihedral restraint directive\n",$opt_N);
printf STDERR ("          -S   enum %9s  S2 restraint directive\n",$opt_N);



if ($opt_h) {
print STDERR "
Restraints can be specified in files with the specified fromats and/or
with directives.
Dihedral directive format: dihname1.value1.force1,dihname2.value2.force2
Example: OMEGA.180.200

In all files, all characters in a line after a '!' will be ignored.


Distance restraints
-------------------
Distance restraint files should be in X-PLOR format (.mr files on the PDB web site)
Only lines starting with the keyword 'assign' will be considered
Setting the option -u to \"Yes\" causes to omit lines with ambiguous restraints 
(containing the keyword 'or' or the character '#').
All restraints will be included with type' = 1 and fac = 1.0, see the GROMACS manual for details.
Ambiguous restraints will be included with all atoms and the same index, see the GROMACS manual 
for more details.
See the sample NOE restraint file (sample.mr) provided for details on input file format.

S2 (generalized order parameter) restraints
-------------------------------------------
The S2 restraint file should be in the following format:
 resnum1 atomname1 resnum2 atomname2 S2_value S2_error Bonddistance
 Bonddistance can be -1, 0 or a positive real number, it is interpreted as a physical distance only 
 in the 3rd case, for the first two please refer to the additional manual.

";
exit;
}

# Read distance restraint file
# Currently only a 'clean', one-restraint-per-line file with clearly seperable columns is read!!
if ($opt_n ne ""){
    open (NF,"$nmrfile") || die "Cannot open file with experimental NOE restraints: $nmrfile\n";
    $drnum=0;
    $skipped_ambig=0;
    while(<NF>){
        chop;
        $_=~s/^ +//;
        $_=~s/\!.*$//; # Remove comments (after the char '!') 
        next if ($_ !~ /assi/i); # Skip non-restraint lines
        # Skip lines with ambigous restraints if requested
        if (($opt_u) && (($_ =~ / or /) || ($_ =~ /\#/))){
          $skipped_ambig++;
          next;
        }
        $_=~s/HN/ H/g;



        # Code from the author's program 'rstcheck'
        $_=~s/assi[a-z]+ +//ig;
        ($first,$second)=split(/\) *\(/,$_);
        $resnum1=$first;
        $resnum1=~s/^.*resid +([0-9]+) +and.*$/$1/;
        $resnum2=$second;
        $resnum2=~s/^.*resid +([0-9]+) +and.*$/$1/;

        $resvalue=$_; $resvalue=~s/^.*\) ([0-9\. ]+)$/$1/;
        @d=split(/ +/,$resvalue);

       ## deal with ambiguous restraints ##
       $atom1=resolveambig($first); 
       $atom2=resolveambig($second); 


        $N_RN1[$drnum]=$resnum1;
        $N_RN2[$drnum]=$resnum2;

        $N_ATOM1[$drnum]=$atom1;
        $N_ATOM2[$drnum]=$atom2;

        $N_MIN[$drnum]=$d[0]-$d[1];
        $N_MAX[$drnum]=$d[0]+$d[2];

#print STDERR "$_ ($first, $second)\n$resnum1 : $atom1 | $resnum2 $atom2 > $d[0] $d[1] $d[2]\n" if $V;

        $drnum++;
    }
    close(NF);
    if ($opt_u){$ambigout="(excluded $skipped_ambig ambiguous ones)"}
    print STDERR "Read $drnum experimental distance restraints $ambigout\n" if $V;
}#_ if opt_n



# Read S2 restraint file
# Format expected: resnum1 atom1 resnum2 atom2 S2_value S2_error Bond_distance
$en=0;
if ($opt_s ne ""){
    open (SF,"$S2file") || die "Cannot open file with experimental S2 values: $S2file\n";
    $s2num=0;
    while(<SF>){
	chop;
	$_=~s/^ +//;
        $_=~s/\!.*$//; # Remove comments (after the char '!') 
	next if ($_ !~ /[0-9]/); # Skip lines with no data
        ($rn1,$atom1,$rn2,$atom2,$s2,$s2err,$dist)=split(/ +/,$_);
        $S_RN1[$s2num]=$rn1;
        $S_RN2[$s2num]=$rn2;
        $S_ATOM1[$s2num]=$atom1;
        $S_ATOM2[$s2num]=$atom2;
	$S_S2[$s2num]=$s2;
	$S_S2E[$s2num]=$s2err;
        $S_DIST[$s2num]=$dist;
	$s2num++;
    }
    close(SF);
    print STDERR "Read $s2num experimental S2 values from file $S2file\n" if $V;
}#_ if opt_s




$readatoms=0;
$restraintsadded=0;
# Reading and processing topology file(s)
# Restraints will be added before the position restraint include statement
open (TOPIN,"$opt_p") || die "FATAL: Toplogy file $opt_p not found!\n";
if (-e $opt_o){
   print STDERR "Back off! Backing up $opt_o to \#$opt_o\#\n";
   system "mv $opt_o \#$opt_o\#";
}


open (TOPOUT,">$opt_o") || die "FATAL: Unable to create output file $opt_o!\n";
while(<TOPIN>){
    $line=$_;
    if ($_=~/^\[ +atoms +\]/){$readatoms=1}
    elsif ($_=~/^\[/){$readatoms=0}
    if ($readatoms && ($_=~/^ *[0-9]+ +/)){
	$_=~s/^ +//;
       ($anum,$atype,$resnum,$resname,$aname,$cgnr,$charge,$mass,$typeB,$chargeB,$massB)=split(/ +/,$_);

        $ATOMNUM[$resnum]{$aname}=$anum;
	$foundres{$resnum}=1;
    }

    # Adding restraints HERE
    if ($_ =~ /; Include Position restraint file/){

        # NOE distance restraints
        if (($drnum > 0) || ($opt_N ne "")){
           print TOPOUT "[ distance_restraints ]\n";
           print TOPOUT "; ai   aj  type index  type'  low   up1  up2   fac\n";
           # from file
           for ($dn=0; $dn < $drnum; $dn++){
               $index=$dn; # Will do it for now
	       $found_atom1=0;
	       $found_atom2=0;
	       foreach $atom1 (split(/:/,$N_ATOM1[$dn])){
		   $ai=$ATOMNUM[$N_RN1[$dn]]{$atom1};
                   if ($ai ne ""){$found_atom1++}
		   foreach $atom2 (split(/:/,$N_ATOM2[$dn])){
		       $aj=$ATOMNUM[$N_RN2[$dn]]{$atom2};
		       if ($aj ne ""){$found_atom2++}
		       if (($ai ne "") && ($aj ne "")){
                           # Note that the division by 10 is necessary to convert Angstroms (X-PLOR) to nm (GROMACS)!
			   printf TOPOUT (" %4d %4d    1 %4d     1  %5.2f %5.2f %5.2f  1.0 ; %4d%4s:%4d%4s\n",$ai,$aj,$index,$N_MIN[$dn]/10,$N_MAX[$dn]/10,$opt_U, $N_RN1[$dn],$atom1,$N_RN2[$dn],$atom2);
		       }
		   }
	       }
	       if ($found_atom1 == 0){die "None of the atoms of group 1 in NOE restraint $dn (res $N_RN1[$dn] atoms $N_ATOM1[$dn]) found in topology file!\n"}
	       if ($found_atom2 == 0){die "None of the atoms of group 2 in NOE restraint $dn (res $N_RN2[$dn] atoms $N_ATOM2[$dn]) found in topology file!\n"}
	   } 
        print TOPOUT "\n";          
       }

        # S2 restraints
        if (($s2num > 0) || ($opt_S ne "")){
           print TOPOUT "[ s2_restraints ]\n";
           print TOPOUT "; ai   aj  type   S2   S2err  bondlen\n";
           # from file
           for ($s2n=0; $s2n < $s2num; $s2n++){
               $ai=$ATOMNUM[$S_RN1[$s2n]]{$S_ATOM1[$s2n]};
               $aj=$ATOMNUM[$S_RN2[$s2n]]{$S_ATOM2[$s2n]};
               if ($ai eq ""){die "ATOM 1 for S2 restraint $s2n (res $S_RN1[$s2n] atom $S_ATOM1[$s2n]) not found in topology file!"}
               if ($aj eq ""){die "ATOM 2 for S2 restraint $s2n (res $S_RN2[$s2n] atom $S_ATOM2[$s2n]) not found in topology file!"}
               printf TOPOUT (" %4d %4d  1  %7.3f %7.3f %7.3f ; %4d%4s:%4d%4s\n",$ai,$aj,$S_S2[$s2n],$S_S2E[$s2n],$S_DIST[$s2n],$S_RN1[$s2n],$S_ATOM1[$s2n],$S_RN2[$s2n],$S_ATOM2[$s2n]);
           }
           # from directive (to be implemented)
        print TOPOUT "\n";
        }

        # dihedral restraints
        if (($dihenum > 0) || ($opt_D ne "")){
           print TOPOUT "[ dihedrals ]\n";
           print TOPOUT "; ai   aj  ak al  index   dihedral  force\n";
           # from file (to be written)
           for ($dn=0; $dn < $dihenum; $dnn++){
               #$ai=$ATOMNUM[$RN1[$s2n]]{$ATOM1[$s2n]};
               #$aj=$ATOMNUM[$RN2[$s2n]]{$ATOM2[$s2n]};
               #if ($ai eq ""){die "ATOM 1 for S2 restraint $s2n (res $RN1[$s2n] atom $ATOM1[$s2n]) not found in topology file!"}
               #if ($aj eq ""){die "ATOM 2 for S2 restraint $s2n (res $RN2[$s2n] atom $ATOM2[$s2n]) not found in topology file!"}
               #printf TOPOUT (" %4d %4d  1  %7.3f %7.3f %7.3f\n",$ai,$aj,$S2[$s2n],$S2E[$s2n],$DIST[$s2n]);
           }
           # from directive
           foreach $resnum (sort {$a <=> $b} keys %foundres){
	       foreach $directive (split (/:/,$opt_D)){
                   ($name,$value,$force)=split(/\./,$directive);
		   if (($name eq "OMEGA") && ($resnum >1)){
		       $ai=$ATOMNUM[$resnum-1]{CA};
		       $aj=$ATOMNUM[$resnum-1]{C};
		       $ak=$ATOMNUM[$resnum]{N};
		       $al=$ATOMNUM[$resnum]{CA};
		       if (($ai ne "") && ($aj ne "") && ($ak ne "") && ($al ne "")){
			   printf TOPOUT ("%4d %4d %4d %4d  1   %7.3f  %7.3f  6.0; OMEGA for residue $resnum\n",$ai,$aj,$ak,$al,$value,$force);
		       }
		   }
	       }
	   }
        print TOPOUT "\n";
        }



    }

    print TOPOUT $line;
}


 

close(TOPIN);
close(TOPOUT);


# A nice subroutine to resolve atom ambiguities in distance restraints
sub resolveambig{

    my $input=shift;
    my $atom;

       if ($input =~ /or/){
         $name1=$input; $name1=~s/^.*\(name +([A-Z0-9\#]+) +or.*$/$1/;
         if ($name1=~/\#/) {
            $nam=$name1;$nam=~s/\#$//;
	    $atom=$nam."1:".$nam."2:".$nam."3";
	 }
	 else{$atom=$name1}
         $name2=$input; $name2=~s/^.*or +name +([A-Z0-9\#]+) *\).*$/$1/;
         if ($name2=~/\#/) {
            $nam=$name2;$nam=~s/\#$//;
	    $atom.=":".$nam."1:".$nam."2:".$nam."3";
	 }
	 else{$atom.=":".$name2}
       }
       else {
         $name=$input; $name=~s/^.*name +([A-Z0-9\#\*]+) .*$/$1/;
         if ($name=~/\#/) {
            $nam=$name;$nam=~s/\#$//;
	    $atom=$nam."1:".$nam."2:".$nam."3";
         }
         else{
	    $atom=$name;
	 }
       }

    return $atom;

}
